//module:ISA_CTL
//Ver 0.50
//date 20/01/2003
//Kenji Yokomizo <kyokomi@mvi.biglobe.ne.jp>

module  ISA_CTL (CLK,RST,START,END,U_CMD,U_ADR,U_DI,U_DO,U_BUSY,RESET_DEV,XSMEMW,XSMEMR,XIOW,XIOR,ISA_CLK,BALE,AEN,SD_I,SD_O,SD_C,SA,XIOCS6,XMEMR,XMEMW,XIOCHDRY);
input   CLK,RST;
// User IF
input   START; // 1:start
output  END;   // 1:end
input   [1:0] U_CMD; // [MEM(XIO),read(xwrite)] 00:IOW 01:IOR 10:MEMW 11:MEMR   
input   [19:0] U_ADR;
input   [15:0] U_DI;
output  [15:0] U_DO;
output  U_BUSY;
// ISA IF
output RESET_DEV;
output XSMEMW;
output XSMEMR;
output XIOW;
output XIOR;
output ISA_CLK;
output BALE;
input  XIOCHDRY;
output AEN;
input  [15:0] SD_I;
output [15:0] SD_O;
output SD_C;// 1:output 0:input
output [19:0]SA;
input XIOCS6;
output XMEMR;
output XMEMW;
//
//

reg   [2:0] STATE_REG;
parameter [2:0] IDLE_ST =3'b000;
parameter [2:0] T1_ST =3'b001;
parameter [2:0] T2_ST =3'b010;
parameter [2:0] T3_ST =3'b011;
parameter [2:0] T4_ST =3'b100;
parameter [2:0] T5_ST =3'b101;
parameter [2:0] T6_ST =3'b110;
parameter [2:0] TW_ST =3'b111;

reg   [1:0] U_CMD_D; // [MEM(XIO),read(xwrite)] 00:IOW 01:IOR 10:MEMW 11:MEMR   
reg   [19:0] U_ADR_D;
reg   [15:0] U_DI_D;
reg   [15:0] U_DO;
reg   U_BUSY;
reg   START_D,START_ENV;
wire  START_EVN;
reg  XSMEMW;
reg  XSMEMR;
reg  XIOW;
reg  XIOR;
reg  BALE;
reg  AEN;
reg  [15:0]SD_O;
reg  SD_C;
reg  [19:0]SA;
//reg  XIOCS6;
reg  XMEMR;
reg  XMEMW;
reg  END ;
wire RESET_DEV;


assign ISA_CLK = CLK;
assign RESET_DEV = RST;

// User command detect
always@(posedge CLK or posedge RST)
begin
  if (RST==1'b1)
    begin
      U_CMD_D <= 2'b00;
      U_ADR_D <= 20'h00000;
      U_DI_D <= 16'h0000;
      START_D <= 0;
      START_ENV <= 0;
    end
  else
    begin 
      U_CMD_D <= U_CMD;
      U_ADR_D <= U_ADR;
      U_DI_D  <= U_DI;
      START_D <= START;
      if ((START==1'b1)&&(START_D==1'b0))
        START_ENV <= 1;
      else
        START_ENV <= 0;
    end
end      
   
// state machine
always@(posedge CLK or posedge RST)
begin
   if (RST==1'b1) 
      STATE_REG <= IDLE_ST ;
   else 
      case (STATE_REG)
         IDLE_ST:
            if (START_ENV==1'b1) 
               STATE_REG <= T1_ST;
            else
               STATE_REG <= IDLE_ST;
         T1_ST:
            STATE_REG <= T2_ST;
         T2_ST:
            STATE_REG <= T3_ST;
         T3_ST:
            STATE_REG <= T4_ST;
         T4_ST:
            STATE_REG <= TW_ST;
         TW_ST:
            if (XIOCHDRY==1'b1)
              STATE_REG <= T5_ST;
            else
              STATE_REG <= TW_ST;
         T5_ST:
            STATE_REG <= T6_ST;
         T6_ST:
            STATE_REG <= IDLE_ST;

         default :
            STATE_REG <= IDLE_ST;
      endcase
end


always@(posedge CLK or posedge RST)
begin
   if (RST==1'b1) 
     begin
       XSMEMW <= 1'b1;
       XSMEMR <= 1'b1;
       XIOW   <= 1'b1;
       XIOR   <= 1'b1;
       BALE   <= 1'b0;
       AEN    <= 1'b1;
       SD_O   <= 16'b0000;
       SD_C   <= 1'b0;
       SA     <= 20'h00300;
//       XIOCS6 <= 1'b0;
       XMEMR  <= 1'b1;
       XMEMW  <= 1'b1;
       U_DO   <= 16'h0000;
       U_BUSY  <= 1'b0;
       END <= 1'b0;
     end       
   else 
     case (STATE_REG)
       IDLE_ST:
         begin
           XSMEMW <= 1'b1;
           XSMEMR <= 1'b1;
           XIOW   <= 1'b1;
           XIOR   <= 1'b1;
           BALE   <= 1'b0;
           AEN    <= 1'b1;
           SD_O   <= 16'b0000;
           SD_C   <= 1'b0;
           SA     <= U_ADR_D;
//           XIOCS6 <= 1'b0;
           XMEMR  <= 1'b1;
           XMEMW  <= 1'b1;
           END <= 1'b0;
           U_DO   <= U_DO;
           U_BUSY  <= 1'b0;
         end      
       T1_ST:
         begin
           BALE <= 1'b1;
			  AEN <= 1'b0;
           SA <= U_ADR_D;
           SD_O <= U_DI;
           U_BUSY  <= 1'b1;
         end      
       T2_ST:
         begin
           BALE <= 1'b0;
           AEN <= 1'b0;
           U_BUSY  <= 1'b1;
         end
       T3_ST:
         begin
           BALE <= 1'b0;
           U_BUSY  <= 1'b1;
           case (U_CMD_D)
             2'b00:XIOW <= 1'b0;
             2'b01:XIOR <= 1'b0;
             2'b10:XMEMW <= 1'b0;
             2'b11:XMEMR <= 1'b0;
           endcase
           if (U_CMD_D[0]==1'b0)
             SD_C <= 1'b1;
           else
             SD_C <= 1'b0;
         end
       T4_ST:
         begin
           BALE <=1'b0;
           U_BUSY  <= 1'b1;
         end
       TW_ST:
         begin
           BALE <=1'b0;
           U_BUSY  <= 1'b1;
         end
       T5_ST:
         begin
           BALE <=1'b0;
           U_BUSY  <= 1'b1;
         end
       T6_ST:
         begin
           BALE <=1'b0;
			  AEN <= 1'b1;
           SD_C <= 1'b0;
           XSMEMW <= 1'b1;
           XSMEMR <= 1'b1;
           XIOW   <= 1'b1;
           XIOR   <= 1'b1;
           XMEMR  <= 1'b1;
           XMEMW  <= 1'b1;
           END <= 1'b1;
           if (U_CMD_D[0]==1'b1)
             U_DO <= SD_I;
           else
             U_DO <= U_DO;
           U_BUSY  <= 1'b1;
         end
       default :
         begin
           XSMEMW <= 1'b1;
           XSMEMR <= 1'b1;
           XIOW   <= 1'b1;
           XIOR   <= 1'b1;
           BALE   <= 1'b0;
           AEN    <= 1'b1;
           U_DO   <= 16'h0000;
           SD_O   <= 16'b0000;
           SD_C   <= 1'b0;
           SA     <= 20'h00000;
//           XIOCS6 <= 1'b0;
           XMEMR  <= 1'b1;
           XMEMW  <= 1'b1;
           END <= 1'b0;
           U_BUSY  <= 1'b0;
         end    
    endcase
end


endmodule
